@@ -1,13 +1,11 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -20,15 +18,12 @@
 #define PHP_HTTP_H
 
 #include "php.h"
+#include "zend_types.h" /* for zend_string */
 #include "zend_smart_str.h"
 
-PHPAPI int php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
+PHPAPI void php_url_encode_hash_ex(HashTable *ht, smart_str *formstr,
 				const char *num_prefix, size_t num_prefix_len,
-				const char *key_prefix, size_t key_prefix_len,
-				const char *key_suffix, size_t key_suffix_len,
-				zval *type, char *arg_sep, int enc_type);
-#define php_url_encode_hash(ht, formstr)	php_url_encode_hash_ex((ht), (formstr), NULL, 0, NULL, 0, NULL, 0, NULL)
-
-PHP_FUNCTION(http_build_query);
+				const zend_string *key_prefix,
+				zval *type, const zend_string *arg_sep, int enc_type);
 
 #endif
