@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,7 +14,9 @@
    +----------------------------------------------------------------------+
  */
 
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+# define _GNU_SOURCE
+#endif
 #include "php.h"
 #include "ext/standard/base64.h"
 
@@ -35,40 +35,37 @@ PHPAPI size_t php_url_decode(char *str, size_t len);
 /* {{{ ------- MEMORY stream implementation -------*/
 
 typedef struct {
-	char        *data;
+	zend_string *data;
 	size_t      fpos;
-	size_t      fsize;
-	size_t      smax;
 	int			mode;
 } php_stream_memory_data;
 
 
 /* {{{ */
-static size_t php_stream_memory_write(php_stream *stream, const char *buf, size_t count)
+static ssize_t php_stream_memory_write(php_stream *stream, const char *buf, size_t count)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	assert(ms != NULL);
 
 	if (ms->mode & TEMP_STREAM_READONLY) {
-		return 0;
-	} else if (ms->mode & TEMP_STREAM_APPEND) {
-		ms->fpos = ms->fsize;
+		return (ssize_t) -1;
 	}
-	if (ms->fpos + count > ms->fsize) {
-		char *tmp;
-		if (!ms->data) {
-			tmp = emalloc(ms->fpos + count);
-		} else {
-			tmp = erealloc(ms->data, ms->fpos + count);
+	size_t data_len = ZSTR_LEN(ms->data);
+	if (ms->mode & TEMP_STREAM_APPEND) {
+		ms->fpos = data_len;
+	}
+	if (ms->fpos + count > data_len) {
+		ms->data = zend_string_realloc(ms->data, ms->fpos + count, 0);
+		if (ms->fpos > data_len) {
+			/* zero the bytes added due to seek past end position */
+			memset(ZSTR_VAL(ms->data) + data_len, 0, ms->fpos - data_len);
 		}
-		ms->data = tmp;
-		ms->fsize = ms->fpos + count;
+	} else {
+		ms->data = zend_string_separate(ms->data, 0);
 	}
-	if (!ms->data)
-		count = 0;
 	if (count) {
-		assert(buf!= NULL);
-		memcpy(ms->data+ms->fpos, (char*)buf, count);
+		ZEND_ASSERT(buf != NULL);
+		memcpy(ZSTR_VAL(ms->data) + ms->fpos, (char*) buf, count);
 		ms->fpos += count;
 	}
 	return count;
@@ -77,22 +74,21 @@ static size_t php_stream_memory_write(php_stream *stream, const char *buf, size_
 
 
 /* {{{ */
-static size_t php_stream_memory_read(php_stream *stream, char *buf, size_t count)
+static ssize_t php_stream_memory_read(php_stream *stream, char *buf, size_t count)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	assert(ms != NULL);
 
-	if (ms->fpos == ms->fsize) {
+	if (ms->fpos >= ZSTR_LEN(ms->data)) {
 		stream->eof = 1;
 		count = 0;
 	} else {
-		if (ms->fpos + count >= ms->fsize) {
-			count = ms->fsize - ms->fpos;
+		if (ms->fpos + count > ZSTR_LEN(ms->data)) {
+			count = ZSTR_LEN(ms->data) - ms->fpos;
 		}
 		if (count) {
-			assert(ms->data!= NULL);
-			assert(buf!= NULL);
-			memcpy(buf, ms->data+ms->fpos, count);
+			ZEND_ASSERT(buf != NULL);
+			memcpy(buf, ZSTR_VAL(ms->data) + ms->fpos, count);
 			ms->fpos += count;
 		}
 	}
@@ -105,11 +101,8 @@ static size_t php_stream_memory_read(php_stream *stream, char *buf, size_t count
 static int php_stream_memory_close(php_stream *stream, int close_handle)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
-	assert(ms != NULL);
-
-	if (ms->data && close_handle && ms->mode != TEMP_STREAM_READONLY) {
-		efree(ms->data);
-	}
+	ZEND_ASSERT(ms != NULL);
+	zend_string_release(ms->data);
 	efree(ms);
 	return 0;
 }
@@ -145,20 +138,14 @@ static int php_stream_memory_seek(php_stream *stream, zend_off_t offset, int whe
 					return 0;
 				}
 			} else {
-				if (ms->fpos + (size_t)(offset) > ms->fsize) {
-					ms->fpos = ms->fsize;
-					*newoffs = -1;
-					return -1;
-				} else {
-					ms->fpos = ms->fpos + offset;
-					*newoffs = ms->fpos;
-					stream->eof = 0;
-					return 0;
-				}
+				stream->eof = 0;
+				ms->fpos = ms->fpos + offset;
+				*newoffs = ms->fpos;
+				return 0;
 			}
 		case SEEK_SET:
-			if (ms->fsize < (size_t)(offset)) {
-				ms->fpos = ms->fsize;
+			if (offset < 0) {
+				ms->fpos = 0;
 				*newoffs = -1;
 				return -1;
 			} else {
@@ -169,15 +156,16 @@ static int php_stream_memory_seek(php_stream *stream, zend_off_t offset, int whe
 			}
 		case SEEK_END:
 			if (offset > 0) {
-				ms->fpos = ms->fsize;
-				*newoffs = -1;
-				return -1;
-			} else if (ms->fsize < (size_t)(-offset)) {
+				ms->fpos = ZSTR_LEN(ms->data) + offset;
+				*newoffs = ms->fpos;
+				stream->eof = 0;
+				return 0;
+			} else if (ZSTR_LEN(ms->data) < (size_t)(-offset)) {
 				ms->fpos = 0;
 				*newoffs = -1;
 				return -1;
 			} else {
-				ms->fpos = ms->fsize + offset;
+				ms->fpos = ZSTR_LEN(ms->data) + offset;
 				*newoffs = ms->fpos;
 				stream->eof = 0;
 				return 0;
@@ -207,7 +195,7 @@ static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb) /
 
 	ssb->sb.st_mode = ms->mode & TEMP_STREAM_READONLY ? 0444 : 0666;
 
-	ssb->sb.st_size = ms->fsize;
+	ssb->sb.st_size = ZSTR_LEN(ms->data);
 	ssb->sb.st_mode |= S_IFREG; /* regular file */
 	ssb->sb.st_mtime = timestamp;
 	ssb->sb.st_atime = timestamp;
@@ -244,21 +232,21 @@ static int php_stream_memory_set_option(php_stream *stream, int option, int valu
 						return PHP_STREAM_OPTION_RETURN_ERR;
 					}
 					newsize = *(size_t*)ptrparam;
-					if (newsize <= ms->fsize) {
+					if (newsize <= ZSTR_LEN(ms->data)) {
+						ms->data = zend_string_truncate(ms->data, newsize, 0);
 						if (newsize < ms->fpos) {
 							ms->fpos = newsize;
 						}
 					} else {
-						ms->data = erealloc(ms->data, newsize);
-						memset(ms->data+ms->fsize, 0, newsize - ms->fsize);
-						ms->fsize = newsize;
+						size_t old_size = ZSTR_LEN(ms->data);
+						ms->data = zend_string_realloc(ms->data, newsize, 0);
+						memset(ZSTR_VAL(ms->data) + old_size, 0, newsize - old_size);
 					}
-					ms->fsize = newsize;
 					return PHP_STREAM_OPTION_RETURN_OK;
 			}
-		default:
-			return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 	}
+
+	return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 }
 /* }}} */
 
@@ -303,10 +291,8 @@ PHPAPI php_stream *_php_stream_memory_create(int mode STREAMS_DC)
 	php_stream *stream;
 
 	self = emalloc(sizeof(*self));
-	self->data = NULL;
+	self->data = ZSTR_EMPTY_ALLOC();
 	self->fpos = 0;
-	self->fsize = 0;
-	self->smax = ~0u;
 	self->mode = mode;
 
 	stream = php_stream_alloc_rel(&php_stream_memory_ops, self, 0, _php_stream_mode_to_str(mode));
@@ -317,24 +303,14 @@ PHPAPI php_stream *_php_stream_memory_create(int mode STREAMS_DC)
 
 
 /* {{{ */
-PHPAPI php_stream *_php_stream_memory_open(int mode, char *buf, size_t length STREAMS_DC)
+PHPAPI php_stream *_php_stream_memory_open(int mode, zend_string *buf STREAMS_DC)
 {
 	php_stream *stream;
 	php_stream_memory_data *ms;
 
 	if ((stream = php_stream_memory_create_rel(mode)) != NULL) {
 		ms = (php_stream_memory_data*)stream->abstract;
-
-		if (mode == TEMP_STREAM_READONLY || mode == TEMP_STREAM_TAKE_BUFFER) {
-			/* use the buffer directly */
-			ms->data = buf;
-			ms->fsize = length;
-		} else {
-			if (length) {
-				assert(buf != NULL);
-				php_stream_write(stream, buf, length);
-			}
-		}
+		ms->data = zend_string_copy(buf);
 	}
 	return stream;
 }
@@ -342,14 +318,10 @@ PHPAPI php_stream *_php_stream_memory_open(int mode, char *buf, size_t length ST
 
 
 /* {{{ */
-PHPAPI char *_php_stream_memory_get_buffer(php_stream *stream, size_t *length STREAMS_DC)
+PHPAPI zend_string *_php_stream_memory_get_buffer(php_stream *stream STREAMS_DC)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
-
-	assert(ms != NULL);
-	assert(length != 0);
-
-	*length = ms->fsize;
+	ZEND_ASSERT(ms != NULL);
 	return ms->data;
 }
 /* }}} */
@@ -368,7 +340,7 @@ typedef struct {
 
 
 /* {{{ */
-static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t count)
+static ssize_t php_stream_temp_write(php_stream *stream, const char *buf, size_t count)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	assert(ts != NULL);
@@ -377,19 +349,20 @@ static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t
 		return -1;
 	}
 	if (php_stream_is(ts->innerstream, PHP_STREAM_IS_MEMORY)) {
-		size_t memsize;
-		char *membuf = php_stream_memory_get_buffer(ts->innerstream, &memsize);
+		zend_off_t pos = php_stream_tell(ts->innerstream);
 
-		if (memsize + count >= ts->smax) {
+		if (pos + count >= ts->smax) {
+			zend_string *membuf = php_stream_memory_get_buffer(ts->innerstream);
 			php_stream *file = php_stream_fopen_temporary_file(ts->tmpdir, "php", NULL);
 			if (file == NULL) {
 				php_error_docref(NULL, E_WARNING, "Unable to create temporary file, Check permissions in temporary files directory.");
 				return 0;
 			}
-			php_stream_write(file, membuf, memsize);
+			php_stream_write(file, ZSTR_VAL(membuf), ZSTR_LEN(membuf));
 			php_stream_free_enclosed(ts->innerstream, PHP_STREAM_FREE_CLOSE);
 			ts->innerstream = file;
 			php_stream_encloses(stream, ts->innerstream);
+			php_stream_seek(ts->innerstream, pos, SEEK_SET);
 		}
 	}
 	return php_stream_write(ts->innerstream, buf, count);
@@ -398,7 +371,7 @@ static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t
 
 
 /* {{{ */
-static size_t php_stream_temp_read(php_stream *stream, char *buf, size_t count)
+static ssize_t php_stream_temp_read(php_stream *stream, char *buf, size_t count)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	size_t got;
@@ -481,8 +454,7 @@ static int php_stream_temp_cast(php_stream *stream, int castas, void **ret)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	php_stream *file;
-	size_t memsize;
-	char *membuf;
+	zend_string *membuf;
 	zend_off_t pos;
 
 	assert(ts != NULL);
@@ -515,8 +487,8 @@ static int php_stream_temp_cast(php_stream *stream, int castas, void **ret)
 	}
 
 	/* perform the conversion and then pass the request on to the innerstream */
-	membuf = php_stream_memory_get_buffer(ts->innerstream, &memsize);
-	php_stream_write(file, membuf, memsize);
+	membuf = php_stream_memory_get_buffer(ts->innerstream);
+	php_stream_write(file, ZSTR_VAL(membuf), ZSTR_LEN(membuf));
 	pos = php_stream_tell(ts->innerstream);
 
 	php_stream_free_enclosed(ts->innerstream, PHP_STREAM_FREE_CLOSE);
@@ -600,7 +572,7 @@ PHPAPI php_stream *_php_stream_temp_create(int mode, size_t max_memory_usage STR
 /* }}} */
 
 /* {{{ _php_stream_temp_open */
-PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, char *buf, size_t length STREAMS_DC)
+PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, const char *buf, size_t length STREAMS_DC)
 {
 	php_stream *stream;
 	php_stream_temp_data *ts;
@@ -621,7 +593,7 @@ PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, char
 /* }}} */
 
 PHPAPI const php_stream_ops php_stream_rfc2397_ops = {
-	php_stream_temp_write, php_stream_temp_read,
+	NULL, php_stream_temp_read,
 	php_stream_temp_close, php_stream_temp_flush,
 	"RFC2397",
 	php_stream_temp_seek,
@@ -643,6 +615,8 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 	int base64 = 0;
 	zend_string *base64_comma = NULL;
 
+	ZEND_ASSERT(mode);
+
 	ZVAL_NULL(&meta);
 	if (memcmp(path, "data:", 5)) {
 		return NULL;
@@ -758,7 +732,7 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 		stream->ops = &php_stream_rfc2397_ops;
 		ts = (php_stream_temp_data*)stream->abstract;
 		assert(ts != NULL);
-		ts->mode = mode && mode[0] == 'r' && mode[1] != '+' ? TEMP_STREAM_READONLY : 0;
+		ts->mode = mode[0] == 'r' && mode[1] != '+' ? TEMP_STREAM_READONLY : 0;
 		ZVAL_COPY_VALUE(&ts->meta, &meta);
 	}
 	if (base64_comma) {
@@ -789,12 +763,3 @@ PHPAPI const php_stream_wrapper php_stream_rfc2397_wrapper =	{
 	NULL,
 	1, /* is_url */
 };
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
